'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jotai = require('jotai');
var reactQuery = require('react-query');

var queryClientAtom = jotai.atom(null);
var getQueryClientAtom = jotai.atom(function (get) {
  return get(queryClientAtom) || new reactQuery.QueryClient();
});

function atomWithQuery(createQuery, equalityFn) {
  if (equalityFn === void 0) {
    equalityFn = Object.is;
  }

  var queryDataAtom = jotai.atom(function (get) {
    var queryClient = get(getQueryClientAtom);
    var options = typeof createQuery === 'function' ? createQuery(get) : createQuery;
    var settlePromise = null;

    var getInitialData = function getInitialData() {
      return typeof options.initialData === 'function' ? options.initialData() : options.initialData;
    };

    var dataAtom = jotai.atom(getInitialData() || new Promise(function (resolve, reject) {
      settlePromise = function settlePromise(data, err) {
        if (err) {
          reject(err);
        } else {
          resolve(data);
        }
      };
    }));

    var setData = function setData() {
      throw new Error('atomWithQuery: setting data without mount');
    };

    var prevData = null;

    var listener = function listener(result) {
      if (result.error) {
        if (settlePromise) {
          settlePromise(null, result.error);
          settlePromise = null;
        } else {
          setData(Promise.reject(result.error));
        }

        return;
      }

      if (result.data === undefined || prevData !== null && equalityFn(prevData, result.data)) {
        return;
      }

      prevData = result.data;

      if (settlePromise) {
        settlePromise(result.data);
        settlePromise = null;
      } else {
        setData(result.data);
      }
    };

    var defaultedOptions = queryClient.defaultQueryObserverOptions(options);

    if (typeof defaultedOptions.staleTime !== 'number') {
      defaultedOptions.staleTime = 1000;
    }

    var observer = new reactQuery.QueryObserver(queryClient, defaultedOptions);
    observer.fetchOptimistic(defaultedOptions).then(listener).catch(function (error) {
      return listener({
        error: error
      });
    });

    dataAtom.onMount = function (update) {
      setData = update;
      var unsubscribe = observer.subscribe(listener);
      return unsubscribe;
    };

    return {
      dataAtom: dataAtom,
      observer: observer
    };
  }, function (get, set, action) {
    switch (action.type) {
      case 'refetch':
        {
          var _get2 = get(queryDataAtom),
              dataAtom = _get2.dataAtom,
              observer = _get2.observer;

          set(dataAtom, new Promise(function () {}));
          var p = observer.refetch({
            cancelRefetch: true
          }).then(function () {});
          return p;
        }
    }
  });
  var queryAtom = jotai.atom(function (get) {
    var _get3 = get(queryDataAtom),
        dataAtom = _get3.dataAtom;

    return get(dataAtom);
  }, function (_get, set, action) {
    return set(queryDataAtom, action);
  });
  return queryAtom;
}

function atomWithInfiniteQuery(createQuery, equalityFn) {
  if (equalityFn === void 0) {
    equalityFn = Object.is;
  }

  var queryDataAtom = jotai.atom(function (get) {
    var queryClient = get(getQueryClientAtom);
    var options = typeof createQuery === 'function' ? createQuery(get) : createQuery;
    var settlePromise = null;

    var getInitialData = function getInitialData() {
      return typeof options.initialData === 'function' ? options.initialData() : options.initialData;
    };

    var dataAtom = jotai.atom(getInitialData() || new Promise(function (resolve, reject) {
      settlePromise = function settlePromise(data, err) {
        if (err) {
          reject(err);
        } else {
          resolve(data);
        }
      };
    }));

    var setData = function setData() {
      throw new Error('atomWithInfiniteQuery: setting data without mount');
    };

    var prevData = null;

    var listener = function listener(result) {
      if (result.error) {
        if (settlePromise) {
          settlePromise(null, result.error);
          settlePromise = null;
        } else {
          setData(Promise.reject(result.error));
        }

        return;
      }

      if (result.data === undefined || prevData !== null && equalityFn(prevData, result.data)) {
        return;
      }

      prevData = result.data;

      if (settlePromise) {
        settlePromise(result.data);
        settlePromise = null;
      } else {
        setData(result.data);
      }
    };

    var defaultedOptions = queryClient.defaultQueryObserverOptions(options);

    if (typeof defaultedOptions.staleTime !== 'number') {
      defaultedOptions.staleTime = 1000;
    }

    var observer = new reactQuery.InfiniteQueryObserver(queryClient, defaultedOptions);
    observer.fetchOptimistic(defaultedOptions).then(listener).catch(function (error) {
      return listener({
        error: error
      });
    });

    dataAtom.onMount = function (update) {
      setData = update;
      var unsubscribe = observer == null ? void 0 : observer.subscribe(listener);
      return unsubscribe;
    };

    return {
      dataAtom: dataAtom,
      observer: observer,
      options: options
    };
  }, function (get, set, action) {
    var _get2 = get(queryDataAtom),
        observer = _get2.observer;

    switch (action.type) {
      case 'refetch':
        {
          void observer.refetch();
          break;
        }

      case 'fetchPreviousPage':
        {
          void observer.fetchPreviousPage();
          break;
        }

      case 'fetchNextPage':
        {
          void observer.fetchNextPage();
          break;
        }
    }
  });
  var queryAtom = jotai.atom(function (get) {
    var _get3 = get(queryDataAtom),
        dataAtom = _get3.dataAtom;

    return get(dataAtom);
  }, function (_get, set, action) {
    return set(queryDataAtom, action);
  });
  return queryAtom;
}

exports.atomWithInfiniteQuery = atomWithInfiniteQuery;
exports.atomWithQuery = atomWithQuery;
exports.getQueryClientAtom = getQueryClientAtom;
exports.queryClientAtom = queryClientAtom;
